#!/usr/bin/python 

## @package gtk_configurator.py
# GTK interface to the configurator
# \author Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>
#
# This program runs the remote help configurator's GTK interface

## @file
# GTK interface to the asssitant

## This file is distributed under the MIT license
LICENSE = """\
Copyright (c) 2008, Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os, sys, ConfigParser, gettext
import pygtk
pygtk.require('2.0')
import gtk, gobject, gtk.glade

import misc

gettext.install('remote-help-assistant')

try:
    gtk.window_set_default_icon_list(gtk.gdk.pixbuf_new_from_file(misc.ASSISTANT_PATH + '/../pixmaps/remote-help-assistant.svg'))
except:
    pass

gtk.glade.bindtextdomain('remote-help-assistant', sys.prefix + '/share/locale/')
gtk.glade.textdomain('remote-help-assistant')

## Glade interface object
interface = gtk.glade.XML(misc.ASSISTANT_PATH + '/configurator.glade')

## Quit the program
def gtk_main_quit(widget, event=None):
    gtk.main_quit()

## IP address has changed
def on_ip_changed(widget):
    interface.get_widget("choice-ip-address").set_sensitive(bool(widget.get_active()))

## Handle the user clicking on the "Help" button
def on_helpbutton_clicked(widget):
    try:
        import gnomevfs
        gnomevfs.url_show("ghelp:" + misc.DOC_PATH + '/manual.xml#rha-configurator')
    except ImportError:
        run_cmd = None

        if os.access("/usr/bin/sensible-browser", os.X_OK):
            run_cmd = 'sensible-browser "%s"'

        try:
            import gconf
            gconf_client = gconf.client_get_default()
            if gconf_client.get_bool('/desktop/gnome/url-handlers/http/enabled'):
                run_cmd = gconf_client.get_string('/desktop/gnome/url-handlers/http/command')
        except:
            pass

        if run_cmd is not None:
            if not os.fork():
                os.setsid()
                os.execl('/bin/sh', '/bin/sh', '-c', 'exec ' + run_cmd % ("ghelp:" + misc.DOC_PATH + '/manual.xml#rha-configurator'))

## Handle the user clicking on the "Save as" button
def on_saveasbutton_clicked(widget):

    filechooser = gtk.FileChooserDialog(
        title=_("Save As"),
        parent=interface.get_widget("main-window"),
        action=gtk.FILE_CHOOSER_ACTION_SAVE,
        buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_OK)
        )
    filechooser.set_current_name("configurator.rha")
    filechooser.set_do_overwrite_confirmation(True)

    if filechooser.run() == gtk.RESPONSE_OK:

        config = ConfigParser.RawConfigParser()

        def set_option(name, value_map):
            value = value_map[interface.get_widget(name).get_active()]
            if value is not None:
                config.set('configurator', name, value)
            return value is not None

        config.add_section('configurator')

        config.set('configurator', 'ssh-key', open(misc.CONFIG_DIR + "/local_rsa_key.pub", 'r').readline().strip()[8:])
        set_option("ssh-key-permissions"  , [ "share", "share control" ])
        set_option("choice-mode"          , [ "force-safe", "default-safe", None, "default-advanced", "force-advanced" ])
        set_option("choice-control"       , [ True, None, False ])
        set_option("choice-firewall"      , [ True, None, False ])
        if set_option("choice-ip-editable", [ None, True, False ]):
            config.set('configurator', 'choice-ip-address', interface.get_widget('choice-ip-address').get_text())

        rha_file = open(filechooser.get_filename(), 'wb')
        print >> rha_file, """#!/usr/bin/remote-help-assistant --config 
#
# This is a configuration file for the Remote Help Assistant
# (https://launchpad.net/remote-help-assistant).
#
# Although this can be executed as a script, it is not recommended:
# encouraging people to run scripts is a bad habit.
#
# This file includes an SSH public key, which must never be tampered
# with.  You should destroy this file if it has been sent over a
# non-tamper-proof link.  In particular:
#
#         NEVER SEND CONFIGURATOR FILES OVER THE INTERNET
"""
        config.write(rha_file)

        interface.get_widget("closebutton").set_label("gtk-close")

    filechooser.destroy()


interface.signal_autoconnect({
        'gtk_main_quit'          : gtk_main_quit,
        'on_ip_changed'          : on_ip_changed,
        'on_helpbutton_clicked'  : on_helpbutton_clicked,
        'on_saveasbutton_clicked': on_saveasbutton_clicked
})

## Main GTK window
window = interface.get_widget("main-window")

## Initialise and add values to a combobox
def populate_combobox(name, values, default):
    combobox = interface.get_widget(name)
    combobox.set_model(gtk.ListStore(gobject.TYPE_STRING))
    cell = gtk.CellRendererText()
    combobox.pack_start(cell, True)
    combobox.add_attribute(cell, 'text', 0)

    for option in values:
        combobox.append_text(option)

    combobox.set_active(default)

populate_combobox("ssh-key-permissions", (
        _("let me see their screen"),
        _("let me control their screen, keyboard and mouse"),
        ), 0)

populate_combobox("choice-mode", (
        _("be required to use the assistant's safe mode"),
        _("use the assistant's safe mode by default"),
        _("have their interface mode left unchanged"),
        _("use the assistant's advanced mode by default"),
        _("always use the assistant's advanced mode"),
        ), 2)

populate_combobox("choice-control", (
        _("share their keyboard and mouse by default"),
        _("have keyboard and mouse setting left unchanged"),
        _("not share share their keyboard and mouse by default"),
        ), 1)

populate_combobox("choice-firewall", (
        _("manage their firewall by default"),
        _("have their firewall setting left unchanged"),
        _("not manage their firewall by default"),
        ), 1)

populate_combobox("choice-ip-editable", (
        _("enter an IP address on their own"),
        _("connect to the following Internet address by default:"),
        _("only connect to the following Internet address:"),
        ), 0)

window.show()
gtk.main()
