#!/usr/bin/python

## @file
# Enable or disable the iptables firewall

## @package fix_iptables
# Enable or disable the iptables firewall
# @author Andrew Sayers <andrew-remote_help@pileofstuff.org>
#
# The program that enables or disables the firewall needs to have root
# privileges, and needs to return success or failure on standard
# output.  That could be done in a bash shell script, but it's best
# not to assume the existence of bash.
#
# The return can't return success or failure in its return value, as
# this is eaten by gksudo.

## This file is distributed under the MIT license
LICENSE = """\
Copyright (c) 2008, Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os, sys, misc

os.close(sys.stderr.fileno())

## IPv4 iptables
IPTABLES = '/sbin/iptables'
## IPv6 iptables
IP6TABLES = '/sbin/iptables'

## Enable the firewall
def enable():
    if \
            os.spawnl(os.P_WAIT, IPTABLES, IPTABLES, '-I',  'INPUT', '-p', 'udp', '-m', 'udp', '--sport', '1900', '-j', 'ACCEPT') or \
            os.spawnl(os.P_WAIT, IPTABLES, IPTABLES, '-I', 'OUTPUT', '-p', 'udp', '-m', 'udp', '--dport', '1900', '-j', 'ACCEPT'):
        return False

    for iptables in (IPTABLES, IP6TABLES):
        if \
                os.spawnl(os.P_WAIT, iptables, iptables, '-I', 'OUTPUT', '-o', 'lo', '-j', 'ACCEPT') or \
                os.spawnl(os.P_WAIT, iptables, iptables, '-I', 'INPUT',  '-i', 'lo', '-j', 'ACCEPT') or \
                os.spawnl(os.P_WAIT, iptables, iptables, '-I', 'OUTPUT', '-p', 'tcp', '--sport', str(misc.REMOTE_PORT), '-j', 'ACCEPT') or \
                os.spawnl(os.P_WAIT, iptables, iptables, '-I', 'INPUT',  '-p', 'tcp', '--dport', str(misc.REMOTE_PORT), '-j', 'ACCEPT') or \
                os.spawnl(os.P_WAIT, iptables, iptables, '-I', 'OUTPUT', '-p', 'udp', '--sport', str(misc.REMOTE_PORT), '-j', 'ACCEPT') or \
                os.spawnl(os.P_WAIT, iptables, iptables, '-I', 'INPUT',  '-p', 'udp', '--dport', str(misc.REMOTE_PORT), '-j', 'ACCEPT'):
            return False

    return True

## Disable the firewall
def disable():
    for iptables in (IPTABLES, IPTABLES, IPTABLES, IP6TABLES, IP6TABLES, IP6TABLES, IPTABLES):
        if \
                os.spawnl(os.P_WAIT, iptables, iptables, '-D',  'INPUT', '1') or \
                os.spawnl(os.P_WAIT, iptables, iptables, '-D', 'OUTPUT', '1'):
            return False

    return True

if 'enable' == sys.argv[1]:
    if enable():
        print "Success"
    else:
        print "Failure"
else:
    if disable():
        print "Success"
    else:
        print "Failure"
