#!/usr/bin/python 

## @package gtk_client.py
# GTK interface
# \author Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>
#
# This program runs the remote help assistant's GTK interface

## @file
# GTK interface to the asssitant

## This file is distributed under the MIT license
LICENSE = """\
Copyright (c) 2008, Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import os, sys
import backend, gtk, gtk_safe, gtk_advanced, misc, configurator_response, debugger

## Debugging and logging
debug = debugger.Debug()

# If called from a shell, it's important that this program not die when the shell window is closed,
# unless the debugging level is high enough to send debugging info to the console
if debugger.LOGLEVEL < 2:

    # The following is based on http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/66012
    # Some modifications to that code have been made, because this isn't actually a daemon (and to suit my programming style)
    try:
        # exit first parent
        if os.fork(): sys.exit(0)
    except OSError, e:
        print >> sys.stderr, "fork #1 failed: %d (%s)" % (e.errno, e.strerror)
        sys.exit(1)

    # decouple from parent environment
    os.setsid()

    # do second fork
    try:
        if os.fork(): sys.exit(0)
    except OSError, e:
        print >> sys.stderr, "fork #2 failed: %d (%s)" % (e.errno, e.strerror)
        sys.exit(1)

    debug.register_fork()

## Back-end instance
backend = backend.Backend()

try:
    gtk.window_set_default_icon_list(gtk.gdk.pixbuf_new_from_file(misc.ASSISTANT_PATH + '/../pixmaps/remote-help-assistant.svg'))
except:
    pass

## Has at least one configurator file been applied?
configurator_applied = False

for rha in backend.get_rha_files():
    if configurator_response.run(rha):
        configurator_applied = True
        backend.apply_rha(rha)

if configurator_applied:
    configurator_response.save()

# GTK assistant
if "safe" == backend.get_default_interface():
    gtk_safe.run(backend)
else:
    gtk_advanced.run(backend)

sys.exit(0)
