#!/usr/bin/python

## @file
# Join the top of the net1 layer to the bottom of the sec2 layer

## @package join_net1_sec2
# Join the top of the net1 layer to the bottom of the sec2 layer
# @author Andrew Sayers <andrew-remote_help@pileofstuff.org>
#
# The net1 layer handles nonsecure communication between assistnts.
# The sec2 layer adds security.  This program is called from
# backend.py, and joins the top of net1 to the bottom of sec2.
#
# This should be called with a single argument: \<ssh_data>
#
# This programs standard input and output should go to the top of the net1 layer

## This file is distributed under the MIT license
LICENSE = """\
Copyright (c) 2008, Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import sys, os, socket, time, signal
import misc, two_way_pipe, debugger

## Debugging and logging
debug = debugger.Debug()

## Bottom of the sec2 layer
sec2_bottom = False

while not sec2_bottom:
    try:
        sec2_bottom = misc.listen_sockets('127.0.0.1', misc.REMOTE_PORT+1)[0]
    except socket.error:
        time.sleep(1)
        debug.log("waited 5 seconds for SSH daemon to start, waiting again...")

debug.log('Trapping SSH client via port', misc.REMOTE_PORT+1, os.getppid())

# Tell the parent process that it can connect now
os.kill(os.getppid(), signal.SIGUSR2)

sec2_bottom = two_way_pipe.From_socket(sec2_bottom, timeout=None)

debug.log('Trapped SSH client')

sec2_bottom.write(sys.argv[1])
sec2_bottom.nonblocking_reads()

sec2_bottom.join(two_way_pipe.From_stdio(), "net1 <-> sec2", verbose=True, conn1_name="sec2 layer", conn2_name="net1 layer")

sec2_bottom = None
