## @package misc
# Miscellany that doesn't fit anywhere else
# \author Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>
#
# \note in this program, constants are in uppercase, variables are lowercase

## @file
# Miscellany that doesn't fit anywhere else

## This file is distributed under the MIT license
LICENSE = """\
Copyright (c) 2008, Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import sys, os, time, socket, debugger

## Debugging and logging
debug = debugger.Debug()

## Port to connect to on the remote host
#
# This is hard-coded because the occasinal benefit of being able to
# set it doesn't seem worth the extra confusion for users.  One port's
# as good as another, right?
REMOTE_PORT = 2112

## Path containing Python programs
ASSISTANT_PATH = sys.path[0]
if '' == ASSISTANT_PATH:
    ASSISTANT_PATH = '.'

## Path containing documentation
DOC_PATH = ASSISTANT_PATH + "/../doc/remote-help-assistant"

## Configuration directory
CONFIG_DIR = os.path.expanduser("~/.remote-help-assistant")

## Output of `bzr version-info` (automatically updated by update-misc.py, DO NOT EDIT)
VERSION_INFO = """
BEGIN VERSION INFO
revision-id: andrew-bazaar@pileofstuff.org-20091011171653-ww1ahcn15296yfcs
date: 2009-10-11 18:16:53 +0100
build-date: 2009-10-11 18:33:44 +0100
revno: 160
branch-nick: 0.2
END VERSION INFO
"""

## Listen on the lowest-numbered available port in the specified range
#
# @param address IP address to listen on
# @param port_lower lowest-numbered port to listen on
# @param port_upper past-the-highest port to listen on
# @param family address family to listen on
#
# If \c port_higher is not specified, only \c port_lower is listened on.
def listen_sockets(address, port_lower, port_upper=None, family=socket.AF_INET):
    if port_upper is None: port_upper = port_lower + 1

    for port in range(port_lower, port_upper):
        for res in socket.getaddrinfo(address, port, family, socket.SOCK_STREAM, 0, socket.AI_PASSIVE):
            address_family, socktype, proto, dummy, sockaddr = res
            try:
                sock = socket.socket(address_family, socktype, proto)
            except socket.error:
                continue
            try:
                sock.bind(sockaddr)
                sock.listen(1)
            except socket.error:
                sock.close()
                continue
            return sock, port
    return None, None

## Simple decorator that logs any functions which take more than 1 second to complete
# 
# @param func function to log
# 
# Some functions can take a long time to run.  Adding this function as
# a decorator will log all instances of the decorated function that
# run for more than 1 second.  This information can be useful during
# debugging and profiling.
def log_slow_functions(func):
    def do_log(*args, **kwargs):
        run_time = time.time()
        ret = func(*args, **kwargs)
        run_time = time.time() - run_time
        if run_time >= 1:
            debug.log('%s took' % func.__name__, run_time, 'seconds to complete, and returned', ret, trace=True, stack_level=1)
        return ret
    return do_log
