#!/usr/bin/python

## @file
# Command called by the assistant's SSH server

## @package ssh_mode
# Pass data back from an SSH client to the assistant
# @author Andrew Sayers <andrew-remote_help@pileofstuff.org>
#
# Part of the connection process involves one side setting up a
# special SSH server, and the other side logging in to it.  Because
# it would be unsafe and unhelpful for the server to give a full
# shell to the client, it instead forces this command to be run.
#
# This command communicates with the local assistant using the pipes
# \c CONFIG_DIR/{from|to}_ssh, and with the remote assistant using the
# file descriptors \c stdin and \c stdout.
#
# \section ssh_mode_pipes Some implementation notes
#
# Ideally, this program would just use \c sendmsg() to pass \c stdin
# and \c stdout to the local assistant.  However, \c sendmsg() isn't
# yet supported in Python, and doesn't seem to be getting much
# attention (http://bugs.python.org/issue1194378).
#
# This program could use pipes or Unix domain sockets - I'm not aware
# of any functional difference between them.  I flipped a coin and
# went with pipes.

## This file is distributed under the MIT license
LICENSE = """\
Copyright (c) 2008, Andrew Sayers <andrew-remote-help-assistant@pileofstuff.org>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
"""

import sys, os
import two_way_pipe, vnc_interaction, misc, debugger

## Debugging and logging
debug = debugger.Debug()

if debugger.LOGLEVEL < 2:
    sys.stderr = debugger.set_err_out(misc.CONFIG_DIR + "/send_with_bug_report.txt")
    debug.log("Starting with config dir:", misc.CONFIG_DIR)
    debug.log('revision data:', misc.VERSION_INFO)

# With debugger.LOGLEVEL < 2, the report file is truncated by the main program.  Otherwise, we have to do it ourselves
sys.stderr = debugger.set_err_out(misc.CONFIG_DIR + "/send_with_bug_report.txt", debugger.LOGLEVEL >= 2)

debug.log("SSH server shell started")

if os.path.exists(misc.CONFIG_DIR + "/from_ssh"): os.remove(misc.CONFIG_DIR + "/from_ssh")
if os.path.exists(misc.CONFIG_DIR + "/to_ssh")  : os.remove(misc.CONFIG_DIR + "/to_ssh")
os.mkfifo(misc.CONFIG_DIR + "/from_ssh")
os.mkfifo(misc.CONFIG_DIR + "/to_ssh")
os.chmod(misc.CONFIG_DIR + "/from_ssh", 0600)
os.chmod(misc.CONFIG_DIR + "/to_ssh", 0600)

## two-way pipe for communicating with the local backend
backend = two_way_pipe.From_files_rw(misc.CONFIG_DIR + '/to_ssh', misc.CONFIG_DIR + '/from_ssh')
## Two-way pipe with the server
sec3_bottom = None

sec3_bottom, port = vnc_interaction.get_client()

print >> backend, port

# Wait for the backend to signal readiness
backend.blocking_reads()
if "go\n" != backend.readline():
    os.remove(misc.CONFIG_DIR + "/from_ssh")
    os.remove(misc.CONFIG_DIR + "/to_ssh")
    sys.exit(0)

print >> backend, os.getpid()

sec3_bottom = two_way_pipe.From_socket(sec3_bottom)

print >> backend, "going"


sec3_bottom.nonblocking_reads()
backend.nonblocking_reads()

## Value to exit with
result = sec3_bottom.join(two_way_pipe.From_stdio(), 'Main loop (ssh mode)', backend, verbose=True, conn1_name="sec3 layer", conn2_name="sec2 layer")

os.remove(misc.CONFIG_DIR + "/from_ssh")
os.remove(misc.CONFIG_DIR + "/to_ssh")
 
sys.exit(result)
